--[[
Opennet Firmware

Copyright 2010 Rene Ejury <opennet@absorb.it>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

$Id: opennet.lua 5485 2009-11-01 14:24:04Z jow $
]]--


local uci = require "luci.model.uci"
local cursor = uci.cursor()
require "luci.sys"
require "luci.config"
resource = luci.config.main.resourcebase
require("luci.i18n")

function get_wan()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]  
  wan = cursor:get("on-usergw", "opennet_ugw"..count, "wan")
  if not wan then wan = "" end
  luci.http.prepare_content("text/plain")
  luci.http.write([[<div class="ugw-wan-route" name="wan" status="]]..wan..[[" >&#x00A0;</div>]])
  luci.http.write([[<img class='loading_img_small' name="wan_spinner" src=']]..resource..[[/icons/loading.gif' alt=']]..
    luci.i18n.string([[Loading]])..[[' style="display:none;" />]])
end

function update_wan()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]
  luci.sys.exec("/usr/sbin/on_usergateway_check checkWan "..count)
  get_wan()
end

function get_wan_ping()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]
  ping = cursor:get("on-usergw", "opennet_ugw"..count, "ping")
  if not ping then
    ping = ""
  else
    ping = ping.." ms"
  end
  luci.http.prepare_content("text/plain")
  luci.http.write([[<div name="wan" id="cbi-network-lan-ping" >]]..ping..[[</div>]])
  luci.http.write([[<img class='loading_img_small' name="wan_spinner" src=']]..resource..[[/icons/loading.gif' alt=']]..
    luci.i18n.string([[Loading]])..[[' style="display:none;" />]])
end

function get_speed()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]  
  luci.http.prepare_content("text/plain")
  upload = cursor:get("on-usergw", "opennet_ugw"..count, "upload")
  download = cursor:get("on-usergw", "opennet_ugw"..count, "download")
  if upload or download then
    speed_time = os.date("%c", cursor:get("on-usergw", "opennet_ugw"..count, "speed_time"))
    if not upload or upload == "0" then upload = "?" end
    if not download or download == "0" then download = "?" end
    local speed = upload.." kbps / "..download.." kbps"
    local abbr = luci.i18n.stringf(speed_time..[[: upload to Gateway %s kbit/s; download from Gateway %s kbit/s]], upload, download)
    luci.http.write([[<div class="ugw-wan-speed" name="speed" id="cbi-network-lan-speed" ><abbr title="]]
      ..abbr..[[">]]..speed..[[</abbr></div>]])
  end
  luci.http.write([[<img class='loading_img_small' name="speed_spinner" src=']]..resource..[[/icons/loading.gif' alt=']]..
    luci.i18n.string([[Loading]])..[[' style="display:none;" />]])
end

function update_speed()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]
  luci.sys.exec("/usr/sbin/on_usergateway_check checkSpeed "..count)
  get_speed()
end

function get_mtu()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]  
  luci.http.prepare_content("text/plain")
  local v = cursor:get_all("on-usergw", "opennet_ugw"..count)
  if v.mtu then
    mtu_time = os.date("%c", cursor:get("on-usergw", "opennet_ugw"..count, "mtu_time"))
    luci.http.write([[<div class="ugw-mtu" name="mtu" status="]]..v.mtu..[["><abbr title="]]..mtu_time..[[: ]]
      ..luci.i18n.stringf([[(tried/measured) to Gateway: %s/%s from Gateway: %s/%s]], v.mtu_toGW_tried, v.mtu_toGW_actual, v.mtu_fromGW_tried, v.mtu_fromGW_actual)
      ..[[">&#x00A0;&#x00A0;&#x00A0;&#x00A0;</abbr></div>]])
  end
  luci.http.write([[<img class='loading_img_small' name="mtu_spinner" src=']]..resource..[[/icons/loading.gif' alt=']]..
    luci.i18n.string([[Loading]])..[[' style="display:none;" />]])
end

function update_mtu()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]
  luci.sys.exec("/usr/sbin/on_usergateway_check checkMtu "..count)
  get_mtu()
end

function get_vpn()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]  
  luci.http.prepare_content("text/plain")
  local v = cursor:get_all("on-usergw", "opennet_ugw"..count)
  if not v.age then v.age = "" end
  if v.status then
    luci.http.write([[<div id="cbi-network-lan-status" name="vpn" status="]]..v.status..[["><abbr title="]]
      ..luci.i18n.stringf([[tested %s minutes ago]], v.age)
      ..[[">&#x00A0;&#x00A0;&#x00A0;&#x00A0;</abbr></div>]])
  end
  luci.http.write([[<img class='loading_img_small' name="vpn_spinner" src=']]..resource..[[/icons/loading.gif' alt=']]..
    luci.i18n.string([[Loading]])..[[' style="display:none;" />]])
end

function update_vpn()
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]
  cursor:delete("on-usergw", "opennet_ugw"..count, "age")
  cursor:commit("on-usergw")
  cursor:unload("on-usergw")
  luci.sys.exec("/usr/sbin/on_usergateway_check checkVpn "..count)
  get_vpn()
end

function get_name_button()
  local SYSROOT = os.getenv("LUCI_SYSROOT")
  if not SYSROOT then SYSROOT = "" end        -- SYSROOT is only used for local testing (make runhttpd in luci tree)
  local path = luci.dispatcher.context.requestpath
  local count = path[#path]  
  luci.http.prepare_content("text/plain")
  local v = cursor:get_all("on-usergw", "opennet_ugw"..count)
  if v and v.ipaddr and v.ipaddr ~= "" and not nixio.fs.access(SYSROOT.."/var/run/on_usergateway_check") then
    luci.http.write([[<h3 count="]]..count..[["><input class="cbi-button" type="submit" title="]]
      ..luci.i18n.stringf([[Click to switch Forwarding to Gateway %s (IP: %s)]], v.name, v.ipaddr)
      ..[[" name="select_gw" value="]]..v.name..[[" /></h3>]])
  else
    luci.http.write([[<h3 count="]]..count..[["><input class="cbi-button" type="submit" title="]]
      ..luci.i18n.string([[No IP-Address found for Gateway-Name, Gateway cannot be used]])
      ..[[" name="select_gw" value="]]..v.name..[[" disabled="true" /></h3>]])
  end
end

function check_running()
  local SYSROOT = os.getenv("LUCI_SYSROOT")
  if not SYSROOT then SYSROOT = "" end        -- SYSROOT is only used for local testing (make runhttpd in luci tree)
  luci.http.prepare_content("text/plain")
  if nixio.fs.access(SYSROOT.."/var/run/on_usergateway_check") then
    luci.http.write("script running")
  else
    luci.http.write("")
  end
end

function change_hna()
  require('luci.model.opennet.on_hna_check')
  luci.http.prepare_content("text/plain")
  luci.http.write(changeHNA())
end


function upgrade()
  -- set hna_mask if not yet set to preset
  if not cursor:get("on-usergw", "ugwng_hna_mask") then
    local preset_cursor = uci.cursor()
    preset_cursor:set_confdir("/etc/config_presets")
    cursor:set("on-usergw", "ugwng_hna_mask", preset_cursor:get("on-usergw", "ugwng_hna_mask"))
  end

  -- transfer all openvpn-remote-names back to on-usergw and remove openvpn entries
  local count = 1
  local number_of_gateways = 0
  cursor:foreach ("on-usergw", "usergateway", function() number_of_gateways = number_of_gateways + 1 end)
  while count <= number_of_gateways do
    local name = cursor:get("on-usergw", "opennet_ugw"..count, "name")
    if not name or name == "" then
      remote = cursor:get("openvpn", "opennet_ugw"..count, "remote")
      if remote then
        cursor:set("on-usergw", "opennet_ugw"..count, "name", remote)
      end
      cursor:delete("openvpn", "opennet_ugw"..count)
    end
    count = count + 1
  end

  cursor:commit("openvpn")
  cursor:commit("on-usergw")
end